@@ -74,16 +74,19 @@ public:
   GameArea();
   ~GameArea();
 
+  bool HandlesClickFocus() const;
+  bool HandlesKeyFocus() const;
+
   bool RequiresKnobs() const;
 
   void EvaluateThemeName();
   void SetSokoban(const Sokoban& sokoban);
 
-  bool HandlesFocus() const;
-
   void CalcSize();
-
-  void Draw(int x, int y, size_t w, size_t h);
+  void GetDimension(size_t& width, size_t& height);
+  void Layout();
+  void Draw(Lum::OS::DrawInfo* draw,
+            int x, int y, size_t w, size_t h);
 
   bool HandleMouseEvent(const Lum::OS::MouseEvent& event);
   bool HandleKeyEvent(const Lum::OS::KeyEvent& event);
