@@ -54,57 +54,21 @@ static Lum::Def::AppInfo info;
 typedef Lum::Model::StdTable<Collection*>      CollectionsModel;
 typedef Lum::Base::Reference<CollectionsModel> CollectionsModelRef;
 
-class CollectionObject : public Lum::Object
+class CollectionsModelPainter : public Lum::TableCellPainter
 {
-private:
-  Lum::OS::FontRef font;
-  Collection       *collection;
-
 public:
-  CollectionObject()
-  : font(Lum::OS::display->GetFont())
+  void Draw(Lum::OS::DrawInfo* draw,
+            int x, int y, size_t width, size_t height) const
   {
-    // no code
-  }
-
-  void SetCollection(Collection* collection)
-  {
-    this->collection=collection;
-  }
-
-  void CalcSize()
-  {
-    minWidth=0;
-    minHeight=font->height;
-
-    width=minWidth;
-    height=minHeight;
-
-    Object::CalcSize();
-  }
-
-  void Draw(int x, int y, size_t w, size_t h)
-  {
-    Object::Draw(x,y,w,h);
-
-    if (!OIntersect(x,y,w,h)) {
-      return;
-    }
-
-    /* --- */
-
-    Lum::OS::DrawInfo *draw=GetDrawInfo();
-
-    draw->PushFont(font);
-    draw->PushForeground(parent->GetTextColor(draw));
+    const Collection* collection=dynamic_cast<const CollectionsModel*>(GetModel())->GetEntry(GetRow());
 
     //
     // Name of collection
     //
 
-    draw->DrawString(this->x,
-                     this->y+font->ascent,
-                     collection->GetTitle());
+    DrawStringLeftAligned(draw,
+                          x,y,width,height,
+                          collection->GetTitle());
 
     //
     //
@@ -122,45 +86,9 @@ public:
             Lum::Base::NumberToWString(collection->GetLevelCount());
     }
 
-    draw->DrawString(this->x+width-font->StringWidth(level),
-                     this->y+font->ascent,
-                     level);
-
-    draw->PopFont();
-    draw->PopForeground();
-  }
-
-  static CollectionObject* Create(bool horizFlex, bool vertFlex)
-  {
-    CollectionObject *object=new CollectionObject();
-
-    object->SetFlex(horizFlex,vertFlex);
-
-    return object;
-  }
-};
-
-class CollectionDataProvider : public CollectionsModel::DataProvider
-{
-private:
-  CollectionObject *object;
-
-public:
-  CollectionDataProvider()
-  : object(CollectionObject::Create(true,false))
-  {
-    // no code
-  }
-
-  ~CollectionDataProvider()
-  {
-    delete object;
-  }
-
-  Lum::Object* GetObject(const CollectionsModel::Iterator& iter, size_t column) const
-  {
-    object->SetCollection(*iter);
-    return object;
+    DrawStringRightAligned(draw,
+                           x,y,width,height,
+                           level);
   }
 };
 
@@ -175,7 +103,7 @@ private:
 public:
   SelectCollection()
   : okAction(new Lum::Model::Action),
-    list(new CollectionsModel(new CollectionDataProvider())),
+    list(new CollectionsModel()),
     selection(new Lum::Model::SingleLineSelection),
     result(NULL)
   {
@@ -196,11 +124,9 @@ public:
     Lum::Model::HeaderRef headerModel;
 
     panel=Lum::VPanel::Create(true,true);
-    panel->SetWidth(Lum::Base::Size::pixel,(GetWindow()->GetParent()->GetWidth()*80)/100);
-    panel->SetHeight(Lum::Base::Size::pixel,(GetWindow()->GetParent()->GetHeight()*70)/100);
 
     headerModel=new Lum::Model::HeaderImpl();
-    headerModel->AddColumn(L"Collection",Lum::Base::Size::stdCharWidth,35,true);
+    headerModel->AddColumn(L"Collection",Lum::Base::Size::stdCharWidth,40,true);
 
     table=new Lum::Table();
     table->SetFlex(true,true);
@@ -208,8 +134,11 @@ public:
     table->SetModel(list);
     table->SetHeaderModel(headerModel);
     table->SetSelection(selection);
-    table->SetDoubleClickAction(okAction);
+    table->SetDefaultAction(okAction);
+    table->SetPainter(new CollectionsModelPainter());
     table->GetTableView()->SetAutoFitColumns(true);
+    table->GetTableView()->SetAutoHSize(true);
+    table->GetTableView()->SetAutoVSize(true);
     panel->Add(table);
 
     return panel;
@@ -278,18 +207,24 @@ public:
 
   Lum::Object* GetContent()
   {
-    Lum::Panel       *panel;
-    Lum::Table       *table;
+    Lum::Panel            *panel;
+    Lum::Table            *table;
+    Lum::Model::HeaderRef headerModel;
 
     panel=Lum::VPanel::Create(true,true);
-    panel->SetWidth(Lum::Base::Size::pixel,(GetWindow()->GetParent()->GetWidth()*80)/100);
-    panel->SetHeight(Lum::Base::Size::pixel,(GetWindow()->GetParent()->GetHeight()*70)/100);
+
+    headerModel=new Lum::Model::HeaderImpl();
+    headerModel->AddColumn(L"Level",Lum::Base::Size::stdCharWidth,10,true);
 
     table=new Lum::Table();
     table->SetFlex(true,true);
+    table->SetHeaderModel(headerModel);
     table->SetModel(list);
     table->SetSelection(selection);
-    table->SetDoubleClickAction(okAction);
+    table->SetDefaultAction(okAction);
+    table->GetTableView()->SetAutoFitColumns(true);
+    table->GetTableView()->SetAutoHSize(true);
+    table->GetTableView()->SetAutoVSize(true);
     panel->Add(table);
 
     return panel;
@@ -336,57 +271,21 @@ public:
 typedef Lum::Model::StdRefTable<Theme,std::vector<Theme> > ThemesModel;
 typedef Lum::Base::Reference<ThemesModel>                  ThemesModelRef;
 
-class ThemeObject : public Lum::Object
+class ThemesModelPainter : public Lum::TableCellPainter
 {
-private:
-  Lum::OS::FontRef font;
-  Theme            theme;
-
 public:
-  ThemeObject()
-  : font(Lum::OS::display->GetFont())
+  void Draw(Lum::OS::DrawInfo* draw,
+            int x, int y, size_t width, size_t height) const
   {
-    // no code
-  }
-
-  void SetTheme(const Theme& theme)
-  {
-    this->theme=theme;
-  }
-
-  void CalcSize()
-  {
-    minWidth=0;
-    minHeight=font->height;
-
-    width=minWidth;
-    height=minHeight;
-
-    Object::CalcSize();
-  }
-
-  void Draw(int x, int y, size_t w, size_t h)
-  {
-    Object::Draw(x,y,w,h);
-
-    if (!OIntersect(x,y,w,h)) {
-      return;
-    }
-
-    /* --- */
-
-    Lum::OS::DrawInfo *draw=GetDrawInfo();
-
-    draw->PushFont(font);
-    draw->PushForeground(parent->GetTextColor(draw));
+    const Theme theme=dynamic_cast<const ThemesModel*>(GetModel())->GetEntry(GetRow());
 
     //
     // Name of theme
     //
 
-    draw->DrawString(this->x,
-                     this->y+font->ascent,
-                     theme.name);
+    DrawStringLeftAligned(draw,
+                          x,y,width,height,
+                          theme.name);
 
     //
     //
@@ -394,46 +293,10 @@ public:
 
     std::wstring size=Lum::Base::NumberToWString(theme.boxSize)+L" pixel";
 
-    draw->DrawString(this->x+width-font->StringWidth(size),
-                     this->y+font->ascent,
-                     size);
-
-    draw->PopFont();
-    draw->PopForeground();
-  }
-
-  static ThemeObject* Create(bool horizFlex, bool vertFlex)
-  {
-    ThemeObject *object=new ThemeObject();
-
-    object->SetFlex(horizFlex,vertFlex);
-
-    return object;
-  }
-};
-
-class ThemeDataProvider : public ThemesModel::DataProvider
-{
-private:
-  ThemeObject *object;
-
-public:
-  ThemeDataProvider()
-  : object(ThemeObject::Create(true,false))
-  {
-    // no code
-  }
-
-  ~ThemeDataProvider()
-  {
-    delete object;
-  }
-
-  Lum::Object* GetObject(const ThemesModel::Iterator& iter, size_t column) const
-  {
-    object->SetTheme(*iter);
-    return object;
-  }
+    DrawStringRightAligned(draw,
+                           x,y,width,height,
+                           size);
+   }
 };
 
 class SelectTheme : public Lum::Dlg::ActionDialog
@@ -447,7 +310,7 @@ private:
 public:
   SelectTheme()
   : okAction(new Lum::Model::Action),
-    list(new ThemesModel(themes,new ThemeDataProvider())),
+    list(new ThemesModel(themes)),
     selection(new Lum::Model::SingleLineSelection),
     result(L"")
   {
@@ -464,8 +327,6 @@ public:
     Lum::Model::HeaderRef headerModel;
 
     panel=Lum::VPanel::Create(true,true);
-    panel->SetWidth(Lum::Base::Size::pixel,(GetWindow()->GetParent()->GetWidth()*80)/100);
-    panel->SetHeight(Lum::Base::Size::pixel,(GetWindow()->GetParent()->GetHeight()*70)/100);
 
     headerModel=new Lum::Model::HeaderImpl();
     headerModel->AddColumn(L"Theme",Lum::Base::Size::stdCharWidth,30,true);
@@ -476,8 +337,11 @@ public:
     table->SetModel(list);
     table->SetHeaderModel(headerModel);
     table->SetSelection(selection);
-    table->SetDoubleClickAction(okAction);
+    table->SetDefaultAction(okAction);
+    table->SetPainter(new ThemesModelPainter());
     table->GetTableView()->SetAutoFitColumns(true);
+    table->GetTableView()->SetAutoHSize(true);
+    table->GetTableView()->SetAutoVSize(true);
     panel->Add(table);
 
     return panel;
